# SPDX-FileCopyrightText: 2022 Scott Shawcroft for Adafruit Industries
# SPDX-License-Identifier: MIT

"""
Simple painting demo that draws on the ESP32-S3 Box display in red for the first
touch and blue for the second concurrent touch.
"""

import bitmaptools
import busio
import board
import displayio
import adafruit_tt21100

# Create library object using our Bus I2C & SPI port
i2c = busio.I2C(board.SCL, board.SDA)
tt = adafruit_tt21100.TT21100(i2c)

# Setup a full screen bitmap that we'll modify based on touches
display = board.DISPLAY
pixels = displayio.Bitmap(display.width, display.height, 4)
palette = displayio.Palette(4)
palette[1] = 0xFF0000
palette[2] = 0x0000FF
tg = displayio.TileGrid(pixels, pixel_shader=palette)
g = displayio.Group()
g.append(tg)
display.show(g)

while True:
    for point in tt.touches:
        # perform transformation to get into display coordinate system!
        y = point["y"]
        x = display.width - point["x"]
        size = point["pressure"] - 20
        bitmaptools.fill_region(
            pixels, x - size, y - size, x + size, y + size, point["id"] + 1
        )



# SPDX-FileCopyrightText: 2017 Scott Shawcroft, written for Adafruit Industries
# SPDX-FileCopyrightText: Copyright (c) 2022 Scott Shawcroft for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""
`adafruit_tt21100`
================================================================================

Basic driver for TT21100 touchscreen drivers


* Author(s): Scott Shawcroft

Implementation Notes
--------------------

**Hardware:**

* `Espressif ESP32-S3 Box <https://www.adafruit.com/product/5290>`_

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the supported boards:
  https://circuitpython.org/downloads
* Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice

"""

__version__ = "0.8.5"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_TT21100.git"

import array
import struct
import time

from adafruit_bus_device.i2c_device import I2CDevice

try:
    from typing import List
except ImportError:
    pass

# This is based on:
# https://github.com/espressif/esp-box/blob/master/components/i2c_devices/touch_panel/tt21100.c


class TT21100:
    """
    A driver for the TT21100 capacitive touch sensor.
    """

    def __init__(self, i2c, address=0x24, irq_pin=None):
        self._i2c = I2CDevice(i2c, address)
        self._irq_pin = irq_pin

        self._bytes = bytearray(28)
        self._data_len = array.array("H", [0])

        # Poll for start up.
        with self._i2c as i2c_transaction:
            while self._data_len[0] != 0x0000:
                i2c_transaction.readinto(self._data_len)
                time.sleep(0.02)

    @property
    def touched(self) -> int:
        """Returns the number of touches currently detected"""
        with self._i2c as i2c:
            i2c.readinto(self._data_len)
            # Throw away packets that are header only because they don't actually
            # have any touches
            if self._data_len[0] == 7:
                i2c.readinto(self._bytes, end=7)

        if self._data_len[0] % 10 == 7:
            return self._data_len[0] // 10
        return 0

    @property
    def touches(self) -> List[dict]:
        """
        Returns a list of touchpoint dicts, with 'x' and 'y' containing the
        touch coordinates, and 'id' as the touch # for multitouch tracking
        """
        touchpoints = []
        self._bytes[2] = 0
        while self._bytes[2] != 1:
            with self._i2c as i2c:
                i2c.readinto(self._data_len)
                # Empty queue
                if self._data_len[0] in (0, 2):
                    return []
                i2c.readinto(self._bytes, end=self._data_len[0])

        for i in range(self._data_len[0] // 10):
            touch_id, x, y, pressure = struct.unpack_from(
                "xBHHBxxx", self._bytes, offset=10 * i + 7
            )
            touch_id = touch_id & 0x1F
            point = {"x": x, "y": y, "id": touch_id, "pressure": pressure}

            touchpoints.append(point)
        return touchpoints



# SPDX-FileCopyrightText: 2017 Scott Shawcroft, written for Adafruit Industries
# SPDX-FileCopyrightText: Copyright (c) 2022 Scott Shawcroft for Adafruit Industries
#
# SPDX-License-Identifier: Unlicense

"""
Example for getting touch data from a TT21100 capacitive touch driver over I2C
"""

import time
import busio
import board
import adafruit_tt21100

# Create library object (named "tt") using a Bus I2C port
i2c = busio.I2C(board.SCL, board.SDA)

tt = adafruit_tt21100.TT21100(i2c)

while True:
    # if the screen is being touched print the touches
    if tt.touched:
        print(tt.touches)

    time.sleep(0.15)
